@@ -4,6 +4,7 @@ import os
 import tempfile
 import urllib.request
 from urllib.parse import urlparse, unquote
+from typing import Optional, Dict, Any
 
 import requests
 
@@ -27,8 +28,13 @@ from constants.constants import (ALZHEIMER_SE_TYPE,
 from utils.utils import Utils
 from berrydb.IngestType import IngestType
 from berrydb.IngestFileType import IngestFileType
+from model_garden.model_provider import ModelProvider
+from model_garden.model import BerryDBModel
+from model_garden.model_repo import ModelRepo
+from model_garden.model import Model
 
-logging.basicConfig(level=LOGGING_LEVEL, format="%(asctime)s - %(levelname)s - %(message)s")
+logging.basicConfig(level=LOGGING_LEVEL,
+                    format="%(asctime)s - %(levelname)s - %(message)s")
 
 
 logger = logging.getLogger(__name__)
@@ -58,7 +64,7 @@ class Database:
     def settings(self, settings: Settings | str) -> Settings:
         """
         .. note::
-            `Refer Settings </settings.html>`_ to learn how to create and save settings
+            Refer to the :ref:`Settings <settings>` documentation to learn how to create and save settings.
         .. #end
 
         **Example:**
@@ -81,7 +87,8 @@ class Database:
             self.__settings = settings
             self.__settings_name = None
         else:
-            Utils.print_error_and_exit("settings must be an instance of class Settings. import using 'from berrydb.settings import Settings'")
+            Utils.print_error_and_exit(
+                "settings must be an instance of class Settings. import using 'from berrydb.settings import Settings'")
         return self.__settings
 
     def api_key(self):
@@ -167,7 +174,8 @@ class Database:
         try:
             response = requests.get(url, params=params)
             if response.status_code != 200:
-                Utils.handleApiCallFailure(response.json(), response.status_code)
+                Utils.handleApiCallFailure(
+                    response.json(), response.status_code)
             if debug_mode:
                 print("documents result ", response.json())
             return json.loads(response.text)
@@ -175,7 +183,6 @@ class Database:
             print("Failed to fetch document: {}".format(str(e)))
             return []
 
-
     def get_all_documents_with_col_filter(self, col_filter=["*"]):
         """
         The `get_all_documents_with_col_filter` method retrieves documents from the currently connected database while applying a filter to specify which columns should be included in the returned documents. This method is useful for narrowing down the data returned, allowing you to focus on specific fields of interest.
@@ -208,7 +215,8 @@ class Database:
         try:
             response = requests.get(url)
             if response.status_code != 200:
-                Utils.handleApiCallFailure(response.json(), response.status_code)
+                Utils.handleApiCallFailure(
+                    response.json(), response.status_code)
             if debug_mode:
                 print("documents result ", response.json())
             # return response.json()
@@ -255,7 +263,8 @@ class Database:
         """
 
         from urllib.parse import quote
-        url = bdb_constants.BASE_URL + document_by_id_url.format(quote(document_id))
+        url = bdb_constants.BASE_URL + \
+            document_by_id_url.format(quote(document_id))
         params = {
             "apiKey": self.__api_key,
             "bucket": self.__bucket_name,
@@ -282,7 +291,8 @@ class Database:
                 print("docById result ", jsonRes)
             return jsonRes
         except Exception as e:
-            print("Failed to fetch document by id {} : {}".format(document_id, str(e)))
+            print("Failed to fetch document by id {} : {}".format(
+                document_id, str(e)))
             return ""
 
     def query(self, query: str):
@@ -328,9 +338,11 @@ class Database:
         headers = Utils.get_headers(self.__api_key)
 
         try:
-            response = requests.post(url, json=payload, headers=headers, params=params)
+            response = requests.post(
+                url, json=payload, headers=headers, params=params)
             if response.status_code != 200:
-                Utils.handleApiCallFailure(response.json(), response.status_code)
+                Utils.handleApiCallFailure(
+                    response.json(), response.status_code)
             if debug_mode:
                 print("query result ", response.json())
             return json.loads(response.text)
@@ -353,7 +365,8 @@ class Database:
         headers = Utils.get_headers(self.__api_key)
 
         try:
-            response = requests.post(url, data=payload, headers=headers, params=params)
+            response = requests.post(
+                url, data=payload, headers=headers, params=params)
             if response.status_code != 200:
                 try:
                     resp_content = response.json()
@@ -432,7 +445,8 @@ class Database:
         """
 
         from urllib.parse import quote
-        url = bdb_constants.BASE_URL + document_by_id_url.format(quote(document_id))
+        url = bdb_constants.BASE_URL + \
+            document_by_id_url.format(quote(document_id))
         params = {
             "apiKey": self.__api_key,
             "bucket": self.__bucket_name,
@@ -446,13 +460,15 @@ class Database:
         try:
             response = requests.delete(url, params=params)
             if response.status_code != 200:
-                Utils.handleApiCallFailure(response.json(), response.status_code)
+                Utils.handleApiCallFailure(
+                    response.json(), response.status_code)
             jsonRes = response.text
             if debug_mode:
                 print("Delete document result ", jsonRes)
             return jsonRes
         except Exception as e:
-            print("Failed to delete document by id {}, reason : {}".format(document_id, str(e)))
+            print("Failed to delete document by id {}, reason : {}".format(
+                document_id, str(e)))
             return ""
 
     # def transcribe(self, video_url: str):
@@ -497,13 +513,15 @@ class Database:
         try:
             response = requests.post(url, headers=headers, data=payload)
             if response.status_code != 200:
-                Utils.handleApiCallFailure(response.json(), response.status_code)
+                Utils.handleApiCallFailure(
+                    response.json(), response.status_code)
             res = response.text
             if debug_mode:
                 print("Youtube transcription result: ", res)
             return res
         except Exception as e:
-            print(f"Failed to get transcription for the youtube url {video_url}, reason : {str(e)}")
+            print(
+                f"Failed to get transcription for the youtube url {video_url}, reason : {str(e)}")
             return ""
 
     def caption(self, image_url: str):
@@ -522,16 +540,18 @@ class Database:
         try:
             response = requests.post(url, headers=headers, data=payload)
             if response.status_code != 200:
-                Utils.handleApiCallFailure(response.json(), response.status_code)
+                Utils.handleApiCallFailure(
+                    response.json(), response.status_code)
             res = response.text
             if debug_mode:
                 print("Caption result: ", res)
             return res
         except Exception as e:
-            print(f"Failed to get caption for the url {image_url}, reason : {str(e)}")
+            print(
+                f"Failed to get caption for the url {image_url}, reason : {str(e)}")
             return ""
 
-    def enable_fts(self, fields = None, override = False):
+    def enable_fts(self, fields=None, override=False):
         """Creates a new full-text search (FTS) index on the database.
 
         Args:
@@ -562,10 +582,12 @@ class Database:
             print("payload:", payload)
 
         try:
-            response = requests.post(url, params=params, data=payload, headers={'Content-Type': 'application/json'})
+            response = requests.post(url, params=params, data=payload, headers={
+                                     'Content-Type': 'application/json'})
 
             if response.status_code != 201:
-                Utils.handleApiCallFailure(response.json(), response.status_code)
+                Utils.handleApiCallFailure(
+                    response.json(), response.status_code)
 
             res = response.json()
             if debug_mode:
@@ -579,7 +601,7 @@ class Database:
 
     def embed(
         self,
-        embedding_api_key:str,
+        embedding_api_key: str,
     ):
         """
         The `embed` method allows users to generate embeddings for the documents in the database using a specified embedding function, often powered by an OpenAI language model. This is useful for tasks such as similarity search, clustering, or any application that requires semantic understanding of the data.
@@ -627,7 +649,8 @@ class Database:
         try:
             response = requests.post(url, headers=headers, data=payload)
             if response.status_code != 200:
-                Utils.handleApiCallFailure(response.json(), response.status_code)
+                Utils.handleApiCallFailure(
+                    response.json(), response.status_code)
             res = response.json()
             if debug_mode:
                 print("Embed result: ", res)
@@ -639,9 +662,9 @@ class Database:
 
     def chat(
         self,
-        llm_api_key:str,
-        question:str,
-        embedding_model_api_key:str | None = None,
+        llm_api_key: str,
+        question: str,
+        embedding_model_api_key: str | None = None,
     ):
         """
         The `chat` method is designed to query a database using a language model (LLM). This method takes a user-defined question and returns an answer generated by the LLM, allowing for interaction with the database in a conversational manner.
@@ -698,7 +721,8 @@ class Database:
         try:
             response = requests.post(url, headers=headers, data=payload)
             if response.status_code != 200:
-                Utils.handleApiCallFailure(response.json(), response.status_code)
+                Utils.handleApiCallFailure(
+                    response.json(), response.status_code)
             res = response.json()
             if debug_mode:
                 print("Database chat result: ", res)
@@ -710,9 +734,9 @@ class Database:
 
     def chat_for_eval(
         self,
-        llm_api_key:str,
-        question:str,
-        embedding_model_api_key:str | None = None,
+        llm_api_key: str,
+        question: str,
+        embedding_model_api_key: str | None = None,
     ) -> dict:
         """
         The `chat_for_eval` method is designed to query a database using a language model (LLM) and also trace the LLM responses for . This method takes a user-defined question and returns an answer generated by the LLM, allowing for interaction with the database in a conversational manner.
@@ -769,7 +793,8 @@ class Database:
         try:
             response = requests.post(url, headers=headers, data=payload)
             if response.status_code != 200:
-                Utils.handleApiCallFailure(response.json(), response.status_code)
+                Utils.handleApiCallFailure(
+                    response.json(), response.status_code)
             res = response.json()
             if debug_mode:
                 print("Database chat result: ", res)
@@ -779,7 +804,7 @@ class Database:
             print(f"{errMsg}, reason : {str(e)}")
             raise e
 
-    def similarity_search(self, llm_api_key:str, query:str):
+    def similarity_search(self, llm_api_key: str, query: str):
         """
         Performs a search of the database and returns a list of documents based on the query and the configured settings
 
@@ -808,8 +833,7 @@ class Database:
         ```
 
         .. seealso::
-            `Refer Settings </settings.html>`_ to learn how to create and save settings
-        .. #end
+            Refer to the :ref:`Settings <settings>` documentation to learn how to create and save settings.
         """
         url = bdb_constants.BERRY_GPT_BASE_URL + bdb_constants.similarity_search_url
         body = {
@@ -903,13 +927,14 @@ class Database:
                 if response_json["success"]:
                     return response_json["message"]
             else:
-                print(f"Failed with ingest PDFs, status code: {response.status_code}")
+                print(
+                    f"Failed with ingest PDFs, status code: {response.status_code}")
         except Exception as e:
             print(f"Failed with ingest PDFs, reason: {e}")
             if debug_mode:
                 raise e
 
-    def add_annotations(self, document_id: str, annotations: list[dict], task:str, annotated_field:str) -> str | None:
+    def add_annotations(self, document_id: str, annotations: list[dict], task: str, annotated_field: str) -> str | None:
         """
         Adds/updates a list of annotations in a specific document in the database.
 
@@ -932,42 +957,48 @@ class Database:
         - `ValueError`: If `annotations` is not a non-empty list or `document_id` is invalid.
         """
         if not isinstance(annotations, list) or not annotations:
-            raise ValueError("annotations must be a non-empty list of dictionaries.")
+            raise ValueError(
+                "annotations must be a non-empty list of dictionaries.")
         if not document_id or not isinstance(document_id, str):
-            raise ValueError("document_id cannot be None or empty and must be a string.")
+            raise ValueError(
+                "document_id cannot be None or empty and must be a string.")
 
-        url = bdb_constants.BASE_URL + bdb_constants.upsert_annotations_url.format(document_id)
+        url = bdb_constants.BASE_URL + \
+            bdb_constants.upsert_annotations_url.format(document_id)
         params = {
             "apiKey": self.__api_key,
             "databaseName": self.__database_name,
         }
 
-        payload = {"annotations": annotations, "task": task, "annotatedField": annotated_field}
+        payload = {"annotations": annotations,
+                   "task": task, "annotatedField": annotated_field}
         payload = json.dumps(payload)
         headers = Utils.get_headers(self.__api_key)
 
         if debug_mode:
-            logger.debug(f"url: {url}")
-            logger.debug(f"params: {params}")
-            logger.debug(f"payload: {payload}")
+            print(f"url: {url}")
+            print(f"params: {params}")
+            print(f"payload: {payload}")
 
         try:
-            response = requests.post(url, data=payload, headers=headers, params=params)
+            response = requests.post(
+                url, data=payload, headers=headers, params=params)
             response_json = response.json()
             if response.status_code != 200:
                 Utils.handleApiCallFailure(response_json, response.status_code)
 
             if debug_mode:
-                logger.debug(f"add_annotations result: {response_json}")
+                print(f"add_annotations result: {response_json}")
 
             if "message" in response_json:
-                logger.info(f"{response_json['message']}")
+                print(f"{response_json['message']}")
                 return response_json["message"]
             else:
-                logger.warning("Successfully added annotations.")
+                print("Successfully added annotations.")
                 return json.dumps(response_json)
         except Exception as e:
-            logger.error(f"Failed to add annotations for document ID '{document_id}', reason: {str(e)}")
+            logger.error(
+                f"Failed to add annotations for document ID '{document_id}', reason: {str(e)}")
             return None
 
     # Sematic Extraction methods
@@ -1023,7 +1054,8 @@ class Database:
             return self.__semantic_extraction_base(extraction_type, json_path, document_ids, None, annotate)
 
         except Exception as e:
-            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
+            logger.exception("Failed to extract semantics for {}, reason: {}".format(
+                extraction_type, str(e)))
             raise e
 
     def medical_ner(self, json_path, document_ids=[], annotate=False):
@@ -1042,7 +1074,8 @@ class Database:
             return self.__semantic_extraction_base(extraction_type, json_path, document_ids, None, annotate)
 
         except Exception as e:
-            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
+            logger.exception("Failed to extract semantics for {}, reason: {}".format(
+                extraction_type, str(e)))
             raise e
 
     def text_summarization(self, json_path, document_ids=[], annotate=False):
@@ -1061,7 +1094,8 @@ class Database:
             return self.__semantic_extraction_base(extraction_type, json_path, document_ids, None, annotate)
 
         except Exception as e:
-            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
+            logger.exception("Failed to extract semantics for {}, reason: {}".format(
+                extraction_type, str(e)))
             raise e
 
     def image_classification(self, json_path, labels, document_ids=[], annotate=False):
@@ -1081,7 +1115,8 @@ class Database:
             return self.__semantic_extraction_base(extraction_type, json_path, document_ids, labels, annotate)
 
         except Exception as e:
-            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
+            logger.exception("Failed to extract semantics for {}, reason: {}".format(
+                extraction_type, str(e)))
             raise e
 
     def image_captioning(self, json_path, document_ids=[], annotate=False):
@@ -1100,7 +1135,8 @@ class Database:
             return self.__semantic_extraction_base(extraction_type, json_path, document_ids, None, annotate)
 
         except Exception as e:
-            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
+            logger.exception("Failed to extract semantics for {}, reason: {}".format(
+                extraction_type, str(e)))
             raise e
 
     def pneumonia_detection(self, json_path, document_ids=[], annotate=False):
@@ -1119,7 +1155,8 @@ class Database:
             return self.__semantic_extraction_base(extraction_type, json_path, document_ids, None, annotate)
 
         except Exception as e:
-            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
+            logger.exception("Failed to extract semantics for {}, reason: {}".format(
+                extraction_type, str(e)))
             raise e
 
     def alzheimer_detection(self, json_path, document_ids=[], annotate=False):
@@ -1138,7 +1175,8 @@ class Database:
             return self.__semantic_extraction_base(extraction_type, json_path, document_ids, None, annotate)
 
         except Exception as e:
-            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
+            logger.exception("Failed to extract semantics for {}, reason: {}".format(
+                extraction_type, str(e)))
             raise e
 
     def fashion(self, json_path, document_ids=[], annotate=False):
@@ -1157,7 +1195,8 @@ class Database:
             return self.__semantic_extraction_base(extraction_type, json_path, document_ids, None, annotate)
 
         except Exception as e:
-            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
+            logger.exception("Failed to extract semantics for {}, reason: {}".format(
+                extraction_type, str(e)))
             raise e
 
     def audio_transcription(self, json_path, document_ids=[], annotate=False):
@@ -1176,7 +1215,8 @@ class Database:
             return self.__semantic_extraction_base(extraction_type, json_path, document_ids, None, annotate)
 
         except Exception as e:
-            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
+            logger.exception("Failed to extract semantics for {}, reason: {}".format(
+                extraction_type, str(e)))
             raise e
 
     def text_classification(self, json_path, labels, document_ids=[], annotate=False):
@@ -1231,13 +1271,15 @@ class Database:
         """
         extraction_type = TEXT_CLASSIFICATION_SE_TYPE
         if not (labels and len(labels)):
-            raise ValueError(f"Labels are required for {extraction_type} to classify the text.")
+            raise ValueError(
+                f"Labels are required for {extraction_type} to classify the text.")
         try:
 
             return self.__semantic_extraction_base(extraction_type, json_path, document_ids, labels, annotate)
 
         except Exception as e:
-            logger.exception("Failed to extract semantics for {}, reason: {}".format(extraction_type, str(e)))
+            logger.exception("Failed to extract semantics for {}, reason: {}".format(
+                extraction_type, str(e)))
             raise e
 
     def __semantic_extraction_base(self, extraction_type, json_path, document_ids=None, labels=None, annotate=False):
@@ -1245,7 +1287,8 @@ class Database:
         if not json_path:
             raise ValueError("JSON path is required")
         if not annotate and not (document_ids and len(document_ids)):
-            raise ValueError("Document IDs are required if you are not annotating the document")
+            raise ValueError(
+                "Document IDs are required if you are not annotating the document")
 
         url = bdb_constants.BASE_URL + SEMANTICS_PREDICT_URL
         if annotate:
@@ -1276,7 +1319,8 @@ class Database:
         if not annotate:
             print("Retrieving predictions for documents with IDs ", document_ids)
         from requests import Response
-        response: Response = requests.post(url, params=params, data=payload, headers=headers)
+        response: Response = requests.post(
+            url, params=params, data=payload, headers=headers)
 
         if response.status_code == 200:
             if not annotate:
@@ -1298,10 +1342,12 @@ class Database:
         }
 
         try:
-            print("Starting to summarize labels for database: ", self.database_name())
+            print("Starting to summarize labels for database: ",
+                  self.database_name())
             response = requests.post(url, headers=headers, json=data)
             if response.status_code != 200:
-                Utils.handleApiCallFailure(response.json(), response.status_code)
+                Utils.handleApiCallFailure(
+                    response.json(), response.status_code)
 
             print("Response:", response.json())
             return response.json()
@@ -1309,7 +1355,7 @@ class Database:
             print(f"Error while summarizing database: {e}")
             return None
 
-    def transcribe(self, json_path:str, document_ids:list[str]=None):
+    def transcribe(self, json_path: str, document_ids: list[str] = None):
         """
         Transcribes audio for documents using the configured BerryDB model.
 
@@ -1339,7 +1385,7 @@ class Database:
         if json_path is None or len(json_path) == 0:
             raise ValueError("JSON path is required")
 
-        from model_garden.model_repo import ModelRepo, ModelProvider
+        from model_garden.model_repo import ModelRepo
         model_repo = ModelRepo(self.__api_key)
 
         if not document_ids:
@@ -1347,17 +1393,20 @@ class Database:
         else:
             documents = self.get_all_documents(document_ids=document_ids)
 
-        model = model_repo.get_by_name(ModelProvider.BERRYDB_MODEL, 'audio-transcription')
+        model = model_repo.get_by_name(
+            'audio-transcription', ModelProvider.BERRYDB_MODEL)
 
         for doc in documents:
             audio_url = Utils.get_value_from_json_path(doc, json_path)
             if audio_url is None:
-                logger.info(f"Skipping document with id {doc['id']} because json_path {json_path} not found")
+                print(
+                    f"Skipping document with id {doc['id']} because json_path {json_path} not found")
                 continue
             prediction = model.predict({"audioUrl": audio_url})
-            self.add_annotations(doc['id'], [prediction], 'speech transcription', json_path)
+            self.add_annotations(
+                doc['id'], [prediction], 'speech transcription', json_path)
 
-    def analyze_sentiment(self, json_path:str, document_ids:list[str]=None):
+    def analyze_sentiment(self, json_path: str, document_ids: list[str] = None):
         """
         Performs audio sentiment analysis using the configured BerryDB model.
 
@@ -1395,15 +1444,18 @@ class Database:
         else:
             documents = self.get_all_documents(document_ids=document_ids)
 
-        model = model_repo.get_by_name(ModelProvider.BERRYDB_MODEL, 'audio-sentiment-analysis')
+        model = model_repo.get_by_name(
+            'audio-sentiment-analysis', ModelProvider.BERRYDB_MODEL)
 
         for doc in documents:
             audio_url = Utils.get_value_from_json_path(doc, json_path)
             if audio_url is None:
-                print(f"Skipping document with id {doc['id']} because json_path {json_path} not found")
+                print(
+                    f"Skipping document with id {doc['id']} because json_path {json_path} not found")
                 continue
             prediction = model.predict({"audioUrl": audio_url})
-            self.add_annotations(doc['id'], [prediction], 'text classification', json_path)
+            self.add_annotations(
+                doc['id'], [prediction], 'text classification', json_path)
 
     def ingest(self, ingest_type: IngestType, ingest_file_type: IngestFileType, artifacts: list[str]):
         """
@@ -1440,17 +1492,27 @@ class Database:
         # Ingest an Excel sheet by URL
         database.ingest(IngestType.URL, IngestFileType.XLSX, ["https://example.com/data.xlsx"])
         ```
+
+        .. seealso::
+            For details on the enums used, see:
+            - :ref:`IngestType <ingesttype>`
+            - :ref:`IngestFileType <ingestfiletype>`
+        .. #end
+
         ---
         """
         # Validate input combinations
         if ingest_type == IngestType.URL:
             if ingest_file_type not in [IngestFileType.PDF, IngestFileType.XLSX, IngestFileType.HTML]:
-                raise ValueError("URL ingest only supports PDF, XLSX, and HTML file types")
+                raise ValueError(
+                    "URL ingest only supports PDF, XLSX, and HTML file types")
         elif ingest_type == IngestType.FILE:
             if ingest_file_type not in [IngestFileType.PDF, IngestFileType.XLSX]:
-                raise ValueError("File ingest only supports PDF and XLSX file types")
+                raise ValueError(
+                    "File ingest only supports PDF and XLSX file types")
         else:
-            raise ValueError("Invalid ingest_type. Must be IngestType.URL or IngestType.FILE")
+            raise ValueError(
+                "Invalid ingest_type. Must be IngestType.URL or IngestType.FILE")
 
         if not artifacts:
             raise ValueError("Artifacts list cannot be empty")
@@ -1479,7 +1541,8 @@ class Database:
                         # Derive a stable filename from the URL if possible
                         parsed = urlparse(artifact_url)
                         candidate_name = os.path.basename(parsed.path)
-                        candidate_name = unquote(candidate_name) if candidate_name else None
+                        candidate_name = unquote(
+                            candidate_name) if candidate_name else None
                         suffix = f".{ingest_file_type.value.lower()}"
                         if not candidate_name or not candidate_name.lower().endswith(suffix):
                             candidate_name = None  # ignore mismatched or missing
@@ -1491,9 +1554,11 @@ class Database:
                             temp_files.append(temp_path)
                         else:
                             # Fallback to a safe temporary name
-                            temp_file = tempfile.NamedTemporaryFile(delete=False, suffix=suffix)
+                            temp_file = tempfile.NamedTemporaryFile(
+                                delete=False, suffix=suffix)
                             temp_files.append(temp_file.name)
-                            urllib.request.urlretrieve(artifact_url, temp_file.name)
+                            urllib.request.urlretrieve(
+                                artifact_url, temp_file.name)
                             temp_file.close()
 
                     form_data = {
@@ -1539,13 +1604,15 @@ class Database:
         try:
             if files:
                 # Send multipart form-data with file
-                response = requests.post(url, params=params, data=form_data, files=files)
+                response = requests.post(
+                    url, params=params, data=form_data, files=files)
             else:
                 # Send only form-data (HTML URL case)
                 response = requests.post(url, params=params, data=form_data)
 
             if response.status_code != 200:
-                Utils.handleApiCallFailure(response.json(), response.status_code)
+                Utils.handleApiCallFailure(
+                    response.json(), response.status_code)
 
             # Return human-friendly message instead of raw JSON
             if ingest_type == IngestType.URL:
@@ -1564,11 +1631,108 @@ class Database:
             return "Ingest request accepted. Files are being processed asynchronously. Time may vary based on content length."
 
         except Exception as e:
-            logger.error(f"Failed to ingest {ingest_type.value} {ingest_file_type.value}: {str(e)}")
+            logger.error(
+                f"Failed to ingest {ingest_type.value} {ingest_file_type.value}: {str(e)}")
             raise
 
+    def annotate(
+        self,
+        json_path: str,
+        *,
+        model_name: Optional[str] = None,
+        task_name: Optional[str] = None,
+        model_args: Optional[Dict[str, Any]] = None,
+    ) -> None:
+        """
+        Annotate documents using either a model name OR a task name (exclusively).
 
-    def evaluator(self, llm_api_key:str, embedding_api_key:str|None = None, metrics_database_name:str|None = "EvalMetricsDB"):
+        Rules
+        -----
+        - Exactly one of `model_name` or `task_name` must be provided.
+        - If `model_name` is provided, it is used as-is (exact match).
+        - If `task_name` is provided, the backend resolves it to the correct model.
+
+        Parameters
+        ----------
+        **json_path** : (`str`)
+            JSON path to extract data for prediction.
+        **model_name** : (`str`, optional)
+            Exact model name (takes precedence; mutually exclusive with task_name).
+        **task_name** : (`str`, optional)
+            Logical task name (mutually exclusive with model_name).
+        **model_args** : (`dict`, optional)
+            Extra arguments to pass to the model (forwarded if task_name is used).
+
+        Raises
+        ------
+        ValueError
+            If json_path missing, or both/neither of model_name/task_name are provided,
+            or model_args is not dict/None.
+        RuntimeError
+            If the resolved model is not found.
+        """
+        if not json_path:
+            raise ValueError("json_path is required.")
+
+        if model_args is not None and not isinstance(model_args, dict):
+            raise ValueError("model_args must be None or a dictionary.")
+
+        # XOR: exactly one of model_name / task_name
+        if bool(model_name) == bool(task_name):
+            raise ValueError("Provide exactly one of model_name or task_name (not both).")
+
+        # Resolve model name
+        if task_name:
+            resolved_model_name = self.__resolve_model_name_from_backend(task_name, model_args or {})
+        else:
+            resolved_model_name = model_name
+
+        model_repo = ModelRepo(self.__api_key)
+        model: Model = model_repo.get_by_name(resolved_model_name)
+        if model is None:
+            raise RuntimeError(f"Model '{resolved_model_name}' not found.")
+
+        documents = self.get_all_documents()
+        for doc in documents:
+            doc_id = doc["_meta"]["objectId"]
+            data_to_predict = Utils.get_value_from_json_path(doc, json_path)
+            if data_to_predict is None:
+                print(f"Skipping document {doc_id}: json_path '{json_path}' not found.")
+                continue
+
+            if getattr(self, "debug_mode", False):
+                print("data_to_predict:", data_to_predict)
+                print("resolved model:", resolved_model_name)
+
+            args_copy = dict(model_args or {})
+            model_input = data_to_predict
+
+            if isinstance(model, BerryDBModel):  # noqa: F821
+                model_input = {"inputs": data_to_predict}
+                args_copy["label_studio_transform"] = True
+
+            print(f"Fetching prediction for doc {doc_id} via model '{resolved_model_name}'")
+            prediction = model.predict(model_input, model_args=args_copy)
+
+            self.add_annotations(doc_id, [prediction], model.config.name, json_path)
+
+    def __resolve_model_name_from_backend(self, task_name: str, model_args: Dict[str, Any]) -> str:
+        """Call backend to resolve task_name -> concrete model_name."""
+
+        url = bdb_constants.ML_BACKEND_BASE_URL + bdb_constants.resolve_model_name_by_task
+        params = {"apiKey": self.__api_key}
+
+        try:
+            resp = requests.post(url, params=params, json={"task_name": task_name, "model_args": model_args})
+            resp.raise_for_status()
+            data = resp.json()
+            if "model_name" not in data:
+                raise RuntimeError(f"Unable to resolve model from task '{task_name}' at the moment. Please try again later.")
+            return data["model_name"]
+        except requests.RequestException as e:
+            raise RuntimeError(f"Failed to resolve model from task '{task_name}'")
+
+    def evaluator(self, llm_api_key: str, embedding_api_key: str | None = None, metrics_database_name: str | None = "EvalMetricsDB"):
         """
         The `evaluator` method initializes and returns an instance of `BerryDBRAGEvaluator`, configured to assess various metrics for a specific project in BerryDB. This evaluator can be used to measure and track key performance indicators within the database.
 
@@ -1603,11 +1767,11 @@ class Database:
         """
         from evaluator.berrydb_rag_evaluator import BerryDBRAGEvaluator
         return BerryDBRAGEvaluator(
-                    api_key=self.__api_key,
-                    settings_name=self.__settings_name,
-                    settings=self.__settings,
-                    llm_api_key=llm_api_key,
-                    database=self,
-                    embedding_api_key=embedding_api_key,
-                    metrics_database_name=metrics_database_name or "EvalMetricsDB",
-                )
\ No newline at end of file
+            api_key=self.__api_key,
+            settings_name=self.__settings_name,
+            settings=self.__settings,
+            llm_api_key=llm_api_key,
+            database=self,
+            embedding_api_key=embedding_api_key,
+            metrics_database_name=metrics_database_name or "EvalMetricsDB",
+        )
